<?php

namespace Drupal\calculation_fields\Element;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element;
use Drupal\Core\Render\Element\Number;
use Drupal\webform\Utility\WebformElementHelper;
use Symfony\Component\ExpressionLanguage\ExpressionLanguage;

/**
 * Provide a custom form element that display a result of an expression.
 *
 * @FormElement("form_calculation_element")
 */
class FormCalculationElement extends Number {

  /**
   * {@inheritDoc}
   */
  public function getInfo() {
    $info = parent::getInfo();
    $info['#evaluation_fields'] = NULL;
    $info['#process'] = [self::class . '::processFormCalculationElement'];
    $info['#element_validate'] = [self::class . '::validate'];
    return $info;
  }

  /**
   * Validate form math element callback.
   *
   * To avoid disruptive data from the client, every submit evaluate the
   * expression and set the correct/expected value for the element.
   */
  public static function validate(&$element, FormStateInterface $form_state, &$complete_form) {
    $expression = static::buildFieldEvaluation($element["#evaluation_fields"], $form_state);
    $value = NestedArray::getValue($form_state->getValues(), $element['#parents']);
    if (empty($expression) || empty($value)) {
      return;
    }
    $expressionLanguage = new ExpressionLanguage();
    $result = $expressionLanguage->evaluate($expression);
    if ($result !== $value) {
      $form_state->setValue($element["#name"], $result);
    }
  }

  /**
   * {@inheritDoc}
   */
  public static function preRenderNumber($element) {
    $element['#attributes']['type'] = 'number';
    $element['#attributes']['readonly'] = 'readonly';
    Element::setAttributes($element, ['id', 'name', 'value', 'step', 'min', 'max', 'placeholder', 'size', 'fields']);
    static::setAttributes($element, ['form-number', 'js-form-math-element']);
    $element['#attributes']['data-evaluate-expression'] = $element["#evaluation_fields"];
    $element['#attributes']['data-evaluate-fields'] = static::evaluateFields($element["#evaluation_fields"]);
    $element['#attached']['library'][] = 'calculation_fields/calculation_fields';
    return $element;
  }

  /**
   * Process each form calculation element setting custom properties.
   *
   * Set two properties:
   * - data-evaluate-context: used to guarantee that the input will use values,
   * in the correct form context.
   * - data-evaluate-deep-fields: deep fields properties is used to store
   * values of the fields that are not being displayed like in a multistep
   * form in a webform for example.
   *
   * @param array $element
   *   The element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state instance.
   * @param array $form
   *   The form items.
   */
  public static function processFormCalculationElement(array $element, FormStateInterface $form_state, array $form) {
    $element['#attributes']['data-evaluate-context'] = str_replace('_', '-', $form["#form_id"]);
    if (isset($element['#evaluation_decimals'])) {
      $element['#attributes']['data-evaluate-decimals'] = (int) $element['#evaluation_decimals'];
    }
    $dependencyFields = static::evaluateFields($element["#evaluation_fields"]);

    $hasMultipleSteps = $form["progress"]["#current_page"] ?? NULL;

    $dependenciesValues = [];
    foreach ($dependencyFields as $field) {
      if (!empty($form['elements'])) {
        $dependentElement = WebformElementHelper::getElement($form['elements'], $field);
      }
      else {
        $dependentElement = WebformElementHelper::getElement($form, $field);
      }

      if (isset($dependentElement["#webform"]) &&
        ($dependentElement["#webform_parent_key"] !== $hasMultipleSteps || !$dependentElement['#access'])) {
        $dependenciesValues[$field] = $form_state->getValue($field);
      }
    }
    $element['#attributes']['data-evaluate-deep-fields'] = json_encode($dependenciesValues);
    return $element;
  }

  /**
   * Build expression field replacing field name for this value.
   *
   * Example: (:field_name_1 + :field_name2) became (2 + 2).
   *
   * @param string $evaluation
   *   The field expression.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return string
   *   The expression with values.
   */
  public static function buildFieldEvaluation($evaluation, FormStateInterface $form_state) {
    $fields = static::evaluateFields($evaluation);
    $collectedValues = [];
    foreach ($fields as $field) {
      [$fieldName, $defaultValue] = explode('|', $field);
      $value = $form_state->getValue($fieldName);
      if (!empty($value)) {
        $collectedValues[$field] = $value;
      }
      elseif (!is_null($defaultValue)) {
        $collectedValues[$field] = $defaultValue;
      }
    }
    if (count($collectedValues) !== count(array_unique($fields))) {
      return NULL;
    }
    $exp = $evaluation;
    foreach ($collectedValues as $field => $value) {
      $exp = str_replace(':' . $field, $value, $exp);
    }
    return $exp;
  }

  /**
   * Evaluation fields expression getting the field names.
   *
   * @param string $evaluationFields
   *   Evaluation field expression.
   *
   * @return array
   *   An array with the field names.
   */
  public static function evaluateFields($evaluationFields): array {
    $expressionFields = explode(':', $evaluationFields) ?? [];
    $expressionFields = array_filter($expressionFields);
    $fields = [];
    foreach ($expressionFields as $field) {
      $items = explode(' ', $field);
      $items = array_map(function ($item) {
        return preg_replace("/[^a-zA-Z0-9_|]/", "", $item);
      }, $items);
      $filteredItems = array_filter($items, function ($item) {
        return !is_numeric($item) && !empty($item);
      });
      $fields = array_merge($fields, $filteredItems);
    }
    return $fields;
  }

}
